<?php

namespace WpifyWoo\Modules\PacketaShipping;

use WpifyWoo\Abstracts\AbstractModule;
use WpifyWoo\Models\PacketaOrderModel;
use WpifyWoo\Plugin;
use WpifyWoo\Repositories\PacketaOrderRepository;

/**
 * Class PacketaShippingModule
 * @package WpifyWoo\Modules\PacketaShipping
 * @property Plugin $plugin
 */
class PacketaShippingModule extends AbstractModule {

	/**
	 * Packeta API
	 * @var PacketaApi
	 */
	private $packeta_api;
	/**
	 * @var ShippingMethodsGenerator
	 */
	private $shipping_methods_generator;

	/**
	 * PacketaShippingModule constructor.
	 *
	 * @param PacketaApi               $packeta_api PacketaApi.
	 * @param ShippingMethodsGenerator $shipping_methods_generator
	 */
	public function __construct( PacketaApi $packeta_api, ShippingMethodsGenerator $shipping_methods_generator ) {
		$this->packeta_api                = $packeta_api;
		$this->shipping_methods_generator = $shipping_methods_generator;
		parent::__construct();
	}

	/**
	 * Setup
	 * @return void
	 */
	public function setup() {
		require __DIR__ . '/PacketaShippingMethod.php';

		add_filter( 'woocommerce_shipping_methods', array( $this, 'register_packeta_shipping_method' ) );
		add_action( 'woocommerce_after_checkout_validation', array( $this, 'checkout_validation' ), 10, 2 );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_checkout_script' ), 10, 2 );
		add_filter( 'wpify_woo_settings_' . $this->id(), array( $this, 'settings' ) );
		add_action(
				'woocommerce_checkout_update_order_meta',
				array(
						$this,
						'store_pickup_field_update_order_meta',
				),
				15,
				2
		);
		add_action( 'woocommerce_admin_order_data_after_shipping_address', array( $this, 'show_packeta_details' ) );
		add_action( 'woocommerce_email_customer_details', array( $this, 'add_packeta_to_email_notifications' ), 25, 4 );
		add_action( 'woocommerce_checkout_order_processed', array( $this, 'maybe_replace_shipping_address' ), 25, 4 );
		add_action( 'add_meta_boxes', array( $this, 'add_metaboxes' ) );
		add_action( 'admin_init', array( $this, 'maybe_send_to_packeta' ) );
		add_filter( 'bulk_actions-edit-shop_order', [ $this, 'custom_bulk_actions' ], 20, 1 );
		add_filter( 'handle_bulk_actions-edit-shop_order', [ $this, 'handle_custom_bulk_actions' ], 10, 3 );
		add_action( 'init', function () {
			if ( apply_filters( 'wpify_woo_packeta_table_checkout_style', false ) ) {
				add_action( 'woocommerce_review_order_after_shipping', array( $this, 'add_table_branch_id_input' ), 10, 2 );
			} else {
				add_action( 'woocommerce_after_shipping_rate', array( $this, 'add_branch_id_input' ), 10, 2 );
			}
		} );
	}

	/**
	 * Set the module ID.
	 * @return string
	 */
	public function id(): string {
		return 'packeta_shipping';
	}

	/**
	 * Set the module ID.
	 * @return string
	 */
	public function name() {
		return __( 'Packeta Shipping', 'wpify-woo' );
	}


	/**
	 * Add settings
	 * @return array[] Settings.
	 */
	public function settings(): array {
		$settings = array(
				array(
						'id'    => 'api_key',
						'type'  => 'text',
						'label' => __( 'API Key', 'wpify-woo' ),
						'desc'  => __( 'Enter your API key that can be found in <a href="https://client.packeta.com/en/support/">https://client.packeta.com/en/support/</a>', 'wpify-woo' ),
				),
				array(
						'id'    => 'api_password',
						'type'  => 'text',
						'label' => __( 'API Password', 'wpify-woo' ),
						'desc'  => __( 'Enter your API password that can be found in <a href="https://client.packeta.com/en/support/">https://client.packeta.com/en/support/</a>', 'wpify-woo' ),
				),
				array(
						'id'    => 'sender_name',
						'type'  => 'text',
						'label' => __( 'Sender name', 'wpify-woo' ),
						'desc'  => __( 'Enter your sender name that can be found in <a href="https://client.packeta.com/en/senders/">https://client.packeta.com/en/senders/</a>', 'wpify-woo' ),
				),
				array(
						'id'    => 'order_weight',
						'type'  => 'text',
						'label' => __( 'Order weight', 'wpify-woo' ),
						'desc'  => __( 'Enter weight in kg that will be sent to Zasilkovna if the products in order do not have weight specified.', 'wpify-woo' ),
				),
				array(
						'id'    => 'order_default_price',
						'type'  => 'number',
						'label' => __( 'Order default price', 'wpify-woo' ),
						'desc'  => __( 'Enter the default value of the package, if the order price is zero.', 'wpify-woo' ),
				),
				array(
						'id'    => 'replace_shipping_address',
						'type'  => 'switch',
						'label' => __( 'Replace shipping address', 'wpify-woo' ),
						'desc'  => __( 'Enable to replace the WooCommerce shipping address with Packeta address when an order is submitted', 'wpify-woo' ),
				),
				array(
						'id'    => 'display_logo',
						'type'  => 'switch',
						'label' => __( 'Display logo on checkout', 'wpify-woo' ),
						'desc'  => __( 'Enable to display Packetera logo on checkout', 'wpify-woo' ),
				),
				array(
						'id'            => 'logo_type',
						'type'          => 'select',
						'label'         => __( 'Select logo type', 'wpify-woo' ),
						'desc'          => __( 'Select logo to display', 'wpify-woo' ),
						'options'       => [
								[
										'label' => __( 'Packeta CZ', 'wpify-woo' ),
										'value' => 'packeta_cz',
								],
								[
										'label' => __( 'Packeta SK', 'wpify-woo' ),
										'value' => 'packeta_sk',
								],
						],
						'default_value' => 'packeta_cz',
				),
				array(
						'id'            => 'widget_version',
						'type'          => 'select',
						'label'         => __( 'Select widget version', 'wpify-woo' ),
						'desc'          => __( 'Select widget version. For v6 you need to enable External pickup points in you Packeta account under User Information - Allowed branch settings', 'wpify-woo' ),
						'default_value' => 'v5',
						'options'       => [
								[
										'label' => 'v5',
										'value' => 'v5',
								],
								[
										'label' => 'v6',
										'value' => 'v6',
								],
						],
				),
				array(
						'id'      => 'cod_gateways',
						'type'    => 'multiselect',
						'label'   => __( 'Select the COD gateways', 'wpify-woo' ),
						'desc'    => __( 'When sending the order to Packeta, the order will be marked as COD for the selected gateways. This setting does NOT add COD fee to the checkout - to setup the payment gateways fees you can use free plugin WooCommerce Pay for Payment.',
								'wpify-woo' ),
						'options' => $this->plugin->get_woocommerce_integration()->get_gateways(),
						'multi'   => true,
				),
				array(
						'id'      => 'carriers',
						'type'    => 'multiselect',
						'label'   => __( 'Carriers', 'wpify-woo' ),
						'desc'    => __( 'BETA: Select external carriers to use for Zasilkovna. These will appear as shipping methods in WooCommerce - Settings - Shipping. This is BETA feature - some of the carriers do have limitations on what packages they accept, and we cannot guarantee yet this feature will work smoothly on all setups. If you encounter any issue, please write us at support@wpify.io with the description of the issue and the error message you see in the order notes.',
								'wpify-woo' ),
						'options' => $this->get_carriers_select(),
						'multi'   => true,
				),
		);

		return $settings;
	}

	public function get_carriers_select() {
		$select = [];
		foreach ( $this->get_carriers() as $carrier ) {
			$select[] = [
					'label' => $carrier->label,
					'value' => $carrier->id,
			];
		}

		return $select;
	}

	public function get_carriers() {
		return json_decode( file_get_contents( __DIR__ . '/carriers.json' ) );
	}

	/**
	 * Enqeueue the frontend scripts
	 * @throws \Wpify\Core\Exceptions\PluginException
	 */
	public function enqueue_checkout_script() {
		if ( ! is_checkout() ) {
			return;
		}

		if ( 'v6' === $this->get_setting( 'widget_version' ) ) {
			$url = 'https://widget.packeta.com/v6/www/js/library.js';
		} else {
			$url = 'https://widget.packeta.com/www/js/library.js';
		}
		wp_enqueue_script( 'packeta-main', $url, array(), null, true );
		wp_localize_script(
				'packeta-main',
				'packeta',
				array(
						'apiKey' => $this->get_setting( 'api_key' ),
				)
		);

		$assets = $this->plugin->get_webpack_manifest()->get_assets(
				'packeta.js',
				'wpify-woo-packeta',
				array(
						'wpifyWooPacketa' => array(
								'stringNone' => __( 'No branch selected', 'wpify-woo' ),
								'language'   => explode( '_', get_locale() )[0],
						),
				),
				array( 'packeta-main' )
		);
		foreach ( $assets as $key => $asset ) {
			$assets[ $key ]['in_footer'] = true;
		}

		$this->plugin->get_assets()->enqueue_assets( $assets );
	}

	/**
	 * Register the shipping method
	 *
	 * @param array $methods Shipping methods.
	 *
	 * @return array
	 */
	public function register_packeta_shipping_method( array $methods ): array {
		$methods['packeta'] = PacketaShippingMethod::class;

		$selected_carriers = $this->get_setting( 'carriers', true );
		if ( ! empty( $selected_carriers ) ) {
			$carriers = array_filter( $this->get_carriers(), function ( $item ) use ( $selected_carriers ) {
				return in_array( $item->id, $selected_carriers );
			} );
			foreach ( $carriers as $carrier ) {
				$methods[ sprintf( 'packeta_%s', $carrier->id, ) ] = sprintf( 'WpifyWoo\Modules\PacketaShipping\ShippingMethods\%s', $carrier->class );
			}
		}

		return $methods;
	}

	/**
	 * Validate the checkout
	 *
	 * @param array $fields Array of the fields.
	 * @param       $errors
	 */
	public function checkout_validation( $fields, $errors ) {
		$packages = WC()->shipping()->get_packages();

		foreach ( $fields['shipping_method'] as $method ) {
			foreach ( $packages as $i => $package ) {
				if ( isset( $package['rates'][ $method ] ) ) {
					if ( strpos( $package['rates'][ $method ]->id, 'packeta:' ) !== false && empty( $_POST['packeta_id'] ) ) {
						$errors->add( 'validation', __( 'Select Packeta pickup point', 'wpify-woo' ) );
						continue( 2 );
					}
				}
			}
		}
	}

	/**
	 * Save branch ID
	 *
	 * @param int $order_id Order ID.
	 *
	 * @since 1.0.0
	 */
	public function store_pickup_field_update_order_meta( int $order_id ) {
		/** @var  PacketaOrderModel $order */
		$repository = $this->plugin->get_repository( PacketaOrderRepository::class );
		$order      = $repository->get( $order_id );
		if ( $order->is_packeta_shipping() ) {
			$order->set_packeta_id( sanitize_text_field( $_POST['packeta_id'] ) );
			$order->set_packeta_name( sanitize_text_field( $_POST['packeta_name'] ) );
			$order->set_packeta_street( sanitize_text_field( $_POST['packeta_street'] ) );
			$order->set_packeta_city( sanitize_text_field( $_POST['packeta_city'] ) );
			$order->set_packeta_postcode( sanitize_text_field( $_POST['packeta_postcode'] ) );
			$order->set_packeta_url( sanitize_text_field( $_POST['packeta_url'] ) );
			$repository->save( $order );
		}
	}

	/**
	 * Add the button and inputs
	 *
	 * @param $method
	 */
	public function add_branch_id_input( $method ) {
		if ( ! is_checkout() ) {
			return;
		}

		$chosen_shipping_rates = array_filter(
				WC()->session->get( 'chosen_shipping_methods' ),
				function ( $item ) {
					return strpos( $item, 'packeta:' ) !== false;
				}
		);

		if ( empty( $chosen_shipping_rates ) ) {
			return;
		}

		if ( strpos( $method->id, 'packeta:' ) !== false ) { ?>
			<style type="text/css" style="display: none;">
				.wpify-woo-packeta__shipping-method #packeta-point-info {
					display: none;
				}

				.wpify-woo-packeta__shipping-method.wpify-woo-packeta__shipping-method--selected #packeta-point-info {
					display: block;
				}

				.wpify-woo-packeta__shipping-method.wpify-woo-packeta__shipping-method--selected .wpify-woo-packeta__button {
					display: none;
				}

			</style>
			<div class="wpify-woo-packeta__shipping-method">
				<?php if ( $this->get_setting( 'display_logo' ) ) {
					if ( 'packeta_sk' == $this->get_setting( 'logo_type' ) ) {
						$url = $this->plugin->get_asset_url( 'src/Modules/PacketaShipping/assets/images/zasilkovna_sk_logo.png' );
					} else {
						$url = $this->plugin->get_asset_url( 'src/Modules/PacketaShipping/assets/images/zasilkovna_logo.png');
					}
					$url = apply_filters( 'wpify_woo_packeta_logo', $url );
					?>
					<img src="<?php echo $url; ?>" alt="" style="max-width: 100%; height: auto; margin-bottom: 20px;">
				<?php } ?>
				<button type="button" class="zasilkovna-open-widget wpify-woo-packeta__button"
						value=""><?php _e( 'Select pickup point', 'wpify-woo' ); ?></button>
				<input type="hidden" name="packeta_id" id="packeta-point-id">
				<input type="hidden" name="packeta_name" id="packeta-point-name">
				<input type="hidden" name="packeta_city" id="packeta-point-city">
				<input type="hidden" name="packeta_street" id="packeta-point-street">
				<input type="hidden" name="packeta_postcode" id="packeta-point-postcode">
				<input type="hidden" name="packeta_url" id="packeta-point-url">
				<div id="packeta-point-info">
					<div class="packeta-point-info__details"></div>
					<div class="packeta-point-info__change"><a href="#"
															   class="zasilkovna-open-widget"><?php _e( 'Change pickup point', 'wpify-woo' ); ?></a>
					</div>
				</div>
			</div>
			<?php
		}
	}

	/**
	 * Add the button and inputs
	 *
	 * @param $method
	 */
	public function add_table_branch_id_input() {
		if ( ! is_checkout() ) {
			return;
		}


		$chosen_shipping_rates = array_filter(
				WC()->session->get( 'chosen_shipping_methods' ),
				function ( $item ) {
					return strpos( $item, 'packeta:' ) !== false;
				}
		);

		if ( empty( $chosen_shipping_rates ) ) {
			return;
		}

		?>
		<style type="text/css">
			.wpify-woo-packeta__shipping-method #packeta-point-info {
				display: none;
			}

			.wpify-woo-packeta__shipping-method.wpify-woo-packeta__shipping-method--selected #packeta-point-info {
				display: block;
			}

			.wpify-woo-packeta__shipping-method.wpify-woo-packeta__shipping-method--selected .wpify-woo-packeta__button {
				display: none;
			}

		</style>
		<tr id="select-balikovna">
			<th class="balikovna-ico">
				<?php if ( $this->get_setting( 'display_logo' ) ) { ?>
					<img src="<?php echo $this->plugin->get_asset_url( 'src/Modules/PacketaShipping/assets/images/zasilkovna_logo.png' ); ?>" alt="" style="max-width: 100%; height: auto;">
				<?php } ?>
			</th>
			<td>
				<div class="wpify-woo-packeta__shipping-method">

					<button type="button" class="zasilkovna-open-widget wpify-woo-packeta__button"
							value=""><?php _e( 'Select pickup point', 'wpify-woo' ); ?></button>
					<input type="hidden" name="packeta_id" id="packeta-point-id">
					<input type="hidden" name="packeta_name" id="packeta-point-name">
					<input type="hidden" name="packeta_city" id="packeta-point-city">
					<input type="hidden" name="packeta_street" id="packeta-point-street">
					<input type="hidden" name="packeta_postcode" id="packeta-point-postcode">
					<input type="hidden" name="packeta_url" id="packeta-point-url">
					<div id="packeta-point-info">
						<div class="packeta-point-info__details"></div>
						<div class="packeta-point-info__change"><a href="#"
																   class="zasilkovna-open-widget"><?php _e( 'Change pickup point', 'wpify-woo' ); ?></a>
						</div>
					</div>
				</div>
			</td>
		</tr>

		<?php
	}


	/**
	 * Add Packeta details to the email notifications
	 *
	 * @param $order
	 * @param $sent_to_admin
	 * @param $plain_text
	 * @param $email
	 */
	public function add_packeta_to_email_notifications( $order, $sent_to_admin, $plain_text, $email ) {
		$this->show_packeta_details( $order );
	}

	/**
	 * Render the packeta details.
	 *
	 * @param \WC_Order $order WC Order.
	 */
	public function show_packeta_details( $order ) {
		/** @var PacketaOrderModel $order */
		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $order );
		if ( $order->is_packeta_shipping() ) {
			?>
			<h3><?php __('Packeta', 'wpify-woo'); ?></h3>
			<?php echo $order->get_packeta_name(); ?><br/>
			<?php echo $order->get_packeta_street(); ?><br/>
			<?php echo $order->get_packeta_city(); ?><br/>
			<?php echo $order->get_packeta_postcode(); ?><br/>
			<?php
			if ( $order->get_packeta_url() ) {
				printf( '<a href="%s">%s</a><br/>', $order->get_packeta_url(), __( 'Show pickup point details', 'wpify-woo' ) );
			}
			?>
			<br/>
			<?php
		}
	}

	/**
	 * Replace the shipping address if set in plugin settings
	 *
	 * @param int $order_id Order ID.
	 *
	 * @throws \WC_Data_Exception
	 */
	public function maybe_replace_shipping_address( int $order_id ) {
		if ( ! $this->get_setting( 'replace_shipping_address' ) ) {
			return;
		}
		/** @var PacketaOrderModel $order */
		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $order_id );
		if ( ! $order->is_packeta_shipping() ) {
			return;
		}

		$order->get_wc_order()->set_shipping_company( $order->get_packeta_name() );
		$order->get_wc_order()->set_shipping_address_1( $order->get_packeta_street() );
		$order->get_wc_order()->set_shipping_city( $order->get_packeta_city() );
		$order->get_wc_order()->set_shipping_postcode( $order->get_packeta_postcode() );
		$order->get_wc_order()->save();
	}

	public function add_metaboxes() {
		if ( empty( $_GET['post'] ) ) {
			return;
		}
		$post_id = (int) $_GET['post'];
		if ( ! $post_id || 'shop_order' !== get_post_type( $post_id ) ) {
			return;
		}

		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $post_id );
		if ( ! $order->is_packeta_shipping() ) {
			return;
		}

		add_meta_box(
				'wpify-woo-packeta',
				__( 'Packeta shipping', 'wpify-woo' ),
				array(
						$this,
						'render_packeta_metabox',
				),
				'shop_order',
				'side'
		);
	}

	/**
	 * Render the Packeta metabox
	 */
	public function render_packeta_metabox() {
		/** @var PacketaOrderModel $order */
		$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( sanitize_text_field( $_GET['post'] ) );
		if ( $order->get_package_id() ) {
			?>
			<div><strong>Packeta package ID: </strong> <?php echo $order->get_package_id(); ?></div>
			<a class="button" target="_blank"
			   href="<?php echo add_query_arg( array( 'packeta-generate-label' => $order->get_package_id() ), get_edit_post_link( sanitize_text_field( $_GET['post'] ) ) ); ?>"><?php _e( 'Download label', 'wpify-woo' ); ?></a>
		<?php } ?>
		<a class="button"
		   href="<?php echo add_query_arg( array( 'send-to-packeta' => 1 ), get_edit_post_link( sanitize_text_field( $_GET['post'] ) ) ); ?>"><?php $order->get_package_id() ? _e( 'Resend to Packeta', 'wpify-woo' ) : _e( 'Send to Packeta',
					'wpify-woo' ); ?></a>
		<?php
	}

	/**
	 * Maybe send order to packet or generate labels
	 */
	public function maybe_send_to_packeta() {
		if ( ! empty( $_GET['send-to-packeta'] ) && ! empty( $_GET['post'] ) ) {
			$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( sanitize_text_field( $_GET['post'] ) );
			$this->packeta_api->create_packet( $order );
		}
		if ( ! empty( $_GET['packeta-generate-label'] ) ) {
			$date = date( 'Y-m-dH:i:s' );
			header( 'Content-type:application/pdf' );
			header( "Content-Disposition:attachment;filename=labels-{$date}.pdf" );
			$labels = $this->packeta_api->get_packets_labels( $_GET['packeta-generate-label'] );
			if (is_wp_error($labels)) {
				wp_die($labels->get_error_message());
			} else {
				echo $this->packeta_api->get_packets_labels( $_GET['packeta-generate-label'] );
				exit();
			}
		}
	}

	/**
	 * @return PacketaApi
	 */
	public function get_packeta_api(): PacketaApi {
		return $this->packeta_api;
	}

	public function custom_bulk_actions( $actions ) {
		$actions['wpify_woo_packeta_generate_labels'] = __( 'Packeta - Generate labels', 'wpify-woo' );

		return $actions;
	}

	/**
	 * Handle custom bulk action
	 *
	 * @param $redirect_to
	 * @param $action
	 * @param $post_ids
	 *
	 * @return mixed|string
	 */
	public function handle_custom_bulk_actions( $redirect_to, $action, $post_ids ) {
		if ( 'wpify_woo_packeta_generate_labels' === $action ) {
			$package_ids = [];
			foreach ( $post_ids as $post_id ) {
				/** @var PacketaOrderModel $order */
				$order = $this->plugin->get_repository( PacketaOrderRepository::class )->get( $post_id );
				if ( $order->get_package_id() ) {
					$package_ids[] = $order->get_package_id();
				} else {
					$result = $this->packeta_api->create_packet( $order );
					if ( ! is_wp_error( $result ) ) {
						$package_ids[] = $result['package_id'];
					}
				}
			}

			$redirect_to = add_query_arg( [ 'packeta-generate-label' => $package_ids ], admin_url() );
		}

		return $redirect_to;
	}

	/**
	 * @return ShippingMethodsGenerator
	 */
	public function get_shipping_methods_generator(): ShippingMethodsGenerator {
		return $this->shipping_methods_generator;
	}
}
