<?php

namespace WpifyWoo;

use Exception;
use WpifyWoo\Managers\ApiManager;
use WpifyWoo\Managers\ModulesManager;
use WpifyWoo\Managers\PostTypesManager;
use WpifyWoo\Managers\RepositoriesManager;
use WpifyWoo\Managers\TaxonomiesManager;
use WpifyWooDeps\Wpify\Core\Abstracts\AbstractPlugin;
use WpifyWooDeps\Wpify\Core\Exceptions\ContainerInvalidException;
use WpifyWooDeps\Wpify\Core\Exceptions\ContainerNotExistsException;
use WpifyWooDeps\Wpify\Core\Interfaces\RepositoryInterface;
use WpifyWooDeps\Wpify\Core\WebpackManifest;
use WpifyWooDeps\Wpify\Core\WordpressMonologHandler;
use WpifyWooDeps\WpifyCustomFields\WpifyCustomFields;

/**
 * Class Plugin
 *
 * @package Wpify
 */
class Plugin extends AbstractPlugin {

	/** Plugin version */
	public const VERSION = '3.3.1';

	/** Plugin slug name */
	public const PLUGIN_SLUG = 'wpify-woo';

	/** Plugin namespace */
	public const PLUGIN_NAMESPACE = '\\' . __NAMESPACE__;

	/** @var Admin */
	private $admin;

	/** @var PostTypesManager */
	private $post_types_manager;

	/** @var TaxonomiesManager */
	private $taxonomies_manager;

	/** @var RepositoriesManager */
	private $repositories_manager;

	/** @var ApiManager */
	private $api_manager;

	/** @var Assets */
	private $assets;

	/** @var ModulesManager */
	private $modules_manager;

	/** @var WooCommerceIntegration */
	private $woocommerce_integration;

	/** @var WebpackManifest */
	private $webpack_manifest;

	/** @var Logger */
	private $logger;
	/**
	 * @var License
	 */
	private $license;
	/**
	 * @var Premium
	 */
	private $premium;

	private $wcf;

	/**
	 * Plugin constructor.
	 *
	 * @param WebpackManifest $webpack_manifest
	 * @param Admin $admin
	 * @param RepositoriesManager $repositories_manager
	 * @param ApiManager $api_manager
	 * @param PostTypesManager $post_types_manager
	 * @param TaxonomiesManager $taxonomies_manager
	 * @param Assets $assets
	 * @param ModulesManager $modules_manager
	 * @param WooCommerceIntegration $woocommerce_integration
	 * @param Logger $logger
	 * @param License $license
	 * @param Premium $premium
	 * @param WpifyCustomFields $wcf
	 *
	 * @throws ContainerInvalidException
	 * @throws ContainerNotExistsException
	 */
	public function __construct(
		WebpackManifest $webpack_manifest,
		Admin $admin,
		RepositoriesManager $repositories_manager,
		ApiManager $api_manager,
		PostTypesManager $post_types_manager,
		TaxonomiesManager $taxonomies_manager,
		Assets $assets,
		ModulesManager $modules_manager,
		WooCommerceIntegration $woocommerce_integration,
		Logger $logger,
		License $license,
		Premium $premium,
		WpifyCustomFields $wcf
	) {
		$this->admin                   = $admin;
		$this->webpack_manifest        = $webpack_manifest;
		$this->post_types_manager      = $post_types_manager;
		$this->taxonomies_manager      = $taxonomies_manager;
		$this->repositories_manager    = $repositories_manager;
		$this->api_manager             = $api_manager;
		$this->assets                  = $assets;
		$this->modules_manager         = $modules_manager;
		$this->woocommerce_integration = $woocommerce_integration;
		$this->logger                  = $logger;
		$this->license                 = $license;
		$this->premium                 = $premium;
		$this->wcf                     = $wcf;

		parent::__construct();
	}

	public function get_admin(): Admin {
		return $this->admin;
	}

	public function get_repositories_manager(): RepositoriesManager {
		return $this->repositories_manager;
	}

	/**
	 * @param string $class
	 *
	 * @return RepositoryInterface
	 */
	public function get_repository( string $class ) {
		return $this->repositories_manager->get_module( $class );
	}

	public function get_api_manager(): ApiManager {
		return $this->api_manager;
	}

	public function get_api( string $class ) {
		return $this->api_manager->get_module( $class );
	}

	public function get_post_types_manager(): PostTypesManager {
		return $this->post_types_manager;
	}

	public function get_post_type( string $class ) {
		return $this->post_types_manager->get_module( $class );
	}

	public function get_taxonomies_manager(): TaxonomiesManager {
		return $this->taxonomies_manager;
	}

	public function get_taxonomy( string $class ) {
		return $this->taxonomies_manager->get_module( $class );
	}

	public function get_assets(): Assets {
		return $this->assets;
	}

	/**
	 * Print styles in theme
	 *
	 * @param $handles
	 */
	public function print_assets( string ...$handles ) {
		$this->assets->print_assets( $handles );
	}

	/**
	 * @return ModulesManager
	 */
	public function get_modules_manager(): ModulesManager {
		return $this->modules_manager;
	}

	/**
	 * @return ModulesManager
	 */
	public function get_module( $name ) {
		$module = $this->modules_manager->get_module( $name );
		if ( $module ) {
			return $module;
		}

		$path = explode( '\\', $name );

		return $this->modules_manager->get_module( array_pop( $path ) );
	}

	/**
	 * @return WooCommerceIntegration
	 */
	public function get_woocommerce_integration(): WooCommerceIntegration {
		return $this->woocommerce_integration;
	}

	/**
	 * @return WebpackManifest
	 */
	public function get_webpack_manifest(): WebpackManifest {
		return $this->webpack_manifest;
	}

	/**
	 * @return Logger
	 */
	public function get_logger(): Logger {
		return $this->logger;
	}

	public function get_wcf(): WpifyCustomFields {
		return $this->wcf;
	}

	/**
	 * Plugin activation and upgrade
	 *
	 * @param $network_wide
	 *
	 * @return void
	 */
	public function activate( $network_wide ) {
		$wordPressHandler = new WordpressMonologHandler( $this->wpdb, $this->logger->table(), array( 'data' ) );
		$wordPressHandler->set_max_table_rows( 250000 );
		$wordPressHandler->initialize( array() );
	}

	/**
	 * Plugin de-activation
	 *
	 * @param $network_wide
	 *
	 * @return void
	 */
	public function deactivate( $network_wide ) {
	}

	/**
	 * Plugin uninstall
	 *
	 * @return void
	 */
	public function uninstall() {
	}

	/**
	 * @return License
	 */
	public function get_license(): License {
		return $this->license;
	}

	/**
	 * @return Premium
	 */
	public function get_premium(): Premium {
		return $this->premium;
	}

	/**
	 * Method to check if plugin has its dependencies. If not, it silently aborts
	 *
	 * @return bool
	 */
	protected function get_dependencies_exist() {
		return class_exists( 'WooCommerce' );
	}

	/**
	 * @return bool
	 * @throws Exception
	 */
	protected function load_components() {
	}
}
